calculate_simpson_remove_non_standard <- function(pattern) {
    unique_sequences <- unlist(unique(pattern))
    no_isolate_in_group <- list()
    unique_pattern <- unique(pattern)

    # --- ADDED ---- {
    valid_pattern <- unique_pattern[sapply(strsplit(as.character(unique_pattern), split = ""),
          function(x) {
              return (length(x) == 
                      length(which(x %in% c("A","C","T","G") )))
          }
         )]
    # --- ADDED ---- }

    for (seqs in unique_sequences) {
        if (seqs %in% valid_pattern){ # --- ADDED ----
            no_isolate_in_group[[seqs]] <-
                length(which(pattern == seqs))
        }
    }

    d_numerator <- Reduce(f = function(prevs, current) {
        return(prevs + (current * (current - 1)))
    }, x = no_isolate_in_group, init = 0)

    seqs_w_valid_pattern <- pattern[which(pattern %in% valid_pattern)] # --- ADDED ----
    
    d_denominator <- length(seqs_w_valid_pattern) *
        (length(seqs_w_valid_pattern) - 1)

    if (d_numerator == 0 || d_denominator == 0) {
        return(0)
    }

    simpson_index <- 1 - (d_numerator / d_denominator)
    return(list(result = simpson_index))
}

view_simpson_remove_non_standard <- function(result, ...) {
    additional_args <- list(...)[[1]]
    if (exists(result$seqc_name)) {
        seqc <- get(result$seqc_name)
    } else if (!is.null(additional_args[["seqc"]])) {
        seqc <- additional_args[["seqc"]]
    } else {
        stop("Unable to find the sequences used to generate result")
    }
    if (class(seqc) == "processed_seqs") {
        seqc <- seqc$seqc
    }

    results <- result$results
    number_of_result <- length(results)
    isolates_in_groups <- list()
    for (n in seq_len(number_of_result)) {
        isolates_in_groups[[n]] <- list()
        result_levels <- names(results[[n]])
        result_max_depth <- result_levels[length(result_levels)]
        ordered_index <- as.numeric(
            strsplit(result_max_depth, split = ",")[[1]])
        patterns <- minSNPs:::generate_pattern(seqc, ordered_index = c(ordered_index))
        unique_sequences <- unlist(unique(patterns))

        # --- ADDED ---{
        valid_pattern <- unique_sequences[sapply(strsplit(as.character(unique_sequences), split = ""),
              function(x) {
                  return (length(x) == 
                          length(which(x %in% c("A","C","T","G") )))
              }
             )]
        # --- ADDED ---}

        invalid_pattern <- unique_sequences[! unique_sequences %in% valid_pattern] # --- ADDED ---
        groups <- c()
        isolates_in_groups[[n]][["groups"]] <- list()
        for (seqs in unique_sequences) {
            if (seqs %in% valid_pattern) { # --- ADDED ---
                isolates_in_groups[[n]][["groups"]][[seqs]] <-
                    paste(names(which(patterns == seqs)), collapse = ", ")
                
                seq_mod <- seqs
                if ('PY0045-C' %in% names(which(patterns == seqs)) ){
                    seq_mod <- paste(seq_mod, "*", sep = "")
                }
                if ('PY0004-C' %in% names(which(patterns == seqs)) ){
                    seq_mod <- paste(seq_mod, "#", sep = "")
                }
                if ('PY0120-C' %in% names(which(patterns == seqs)) ){
                    seq_mod <- paste(seq_mod, "^", sep = "")
                }
                
                no_MY <- length(names(which(patterns == seqs))[startsWith(names(which(patterns == seqs)), "PY")])
                no_TH <- length(names(which(patterns == seqs))[startsWith(names(which(patterns == seqs)), "PD")])
                no_ID <- length(names(which(patterns == seqs))[startsWith(names(which(patterns == seqs)), "PJ")])
                
                groups <- c(groups, paste(seq_mod, "\n", "MY: ", no_MY, ", ", "TH: ", no_TH, ", ", "ID: ", no_ID))
            }
        }
        isolates_in_groups[[n]][["result"]] <- results[[n]]

        # --- ADDED ---{
        isolates_in_groups[[n]][["N.B.-removed"]] <- names(patterns[which(patterns %in% invalid_pattern)])
        # --- ADDED ---}
        isolates_in_groups[[n]][["N.B.-total removed"]] <- length(names(patterns[which(patterns %in% invalid_pattern)]))
        
        
        isolates_in_groups[[n]][["N.B.-groups breakdown"]] <- paste(groups, collapse = "\n")
    }
    return(isolates_in_groups)
}

calculate_percent_remove_non_standard <- function(pattern, goi) {
    target_seqs <- character()
    unique_pattern <- unique(pattern)

    # --- ADDED ---{
    valid_pattern <- unique_pattern[sapply(strsplit(as.character(unique_pattern), split = ""),
          function(x) {
              return (length(x) == 
                      length(which(x %in% c("A","C","T","G") )))
          }
         )]
    # --- ADDED ---}

    for (isolate in goi) {
        if (is.null(pattern[[isolate]])) {
            stop(paste(isolate, " in group of interest, ",
                "but not found in list of isolates", sep = ""))
        }
        if (pattern[[isolate]] %in% valid_pattern) { # --- ADDED ---
            if (! pattern[[isolate]] %in% target_seqs) {
                target_seqs <- c(target_seqs, pattern[[isolate]])
            }
        }
    }

    # --- ADDED ---{
    accepted_goi <- pattern[goi][pattern[goi] %in% target_seqs]
    accepted_seqs <- pattern[pattern %in% valid_pattern]
    # --- ADDED ---}

    isolate_w_goi_pattern <- length(which(pattern %in% target_seqs))
    failed_to_discriminate <- isolate_w_goi_pattern - length(accepted_goi)
    # --- ADDED ---{
    if (length(accepted_goi) == 0 || (length(accepted_seqs) - length(accepted_goi)) == 0){
        result <- 0
        return(result)
    }
    # --- ADDED ---}
    result <- 1 - (failed_to_discriminate / (length(accepted_seqs) - length(accepted_goi)))

    return(list(result = result))
}


view_percent_remove_non_standard <- function(result, ...) { # nolint
    additional_args <- list(...)[[1]]
    if (exists(result$seqc_name) && !is.null(result$goi)) {
        seqc <- get(result$seqc_name)
    } else if (!is.null(additional_args[["seqc"]]) && !is.null(result$goi)) {
        seqc <- additional_args[["seqc"]]
    } else {
        stop("Unable to find the sequences used to generate result OR GOI")
    }

    if (class(seqc) == "processed_seqs") {
        seqc <- seqc$seqc
    }
    results <- result$results
    number_of_result <- length(results)
    goi <- result$goi
    isolates_in_groups <- list()

    for (n in seq_len(number_of_result)) {
        target_seqs <- character()
        isolates_in_groups[[n]] <- list()
        result_levels <- names(results[[n]])
        result_max_depth <- result_levels[length(result_levels)]
        ordered_index <- as.numeric(
            strsplit(result_max_depth, split = ",")[[1]])
        patterns <- minSNPs:::generate_pattern(seqc, ordered_index = c(ordered_index))
        unique_sequences <- unlist(unique(patterns))
        
        valid_patterns <- unique_sequences[sapply(strsplit(as.character(unique_sequences), split = ""),
          function(x) {
              return (length(x) == 
                      length(which(x %in% c("A","C","T","G") )))
          }
         )]
        
        
        

        isolates_in_groups[[n]][["groups"]] <- list()
        for (isolate in goi) {
            if (patterns[[isolate]] %in% valid_patterns){
                if (! patterns[[isolate]] %in% target_seqs) {
                    target_seqs <- c(target_seqs, patterns[[isolate]])
                }
            }
        }
        
        isolates_w_goi_pattern <- names(patterns)[
            which(patterns %in% target_seqs)]
        failed_to_discriminate <- isolates_w_goi_pattern[
            which(!isolates_w_goi_pattern %in% goi)]

        pattern_failed <- patterns[failed_to_discriminate]

        failed_to_discriminate <- sapply(failed_to_discriminate,
        function(failed_name, pattern) {
            return(paste(failed_name, " (", pattern[[failed_name]], ")",
            sep = ""))
            },
        pattern = pattern_failed, USE.NAMES = FALSE)
        invalid_pattern <- unique_sequences[! unique_sequences %in% valid_patterns] # --- ADDED ---
        for (seqs in unique_sequences) {
            if (seqs %in% valid_patterns){
                if (seqs %in% target_seqs) {
                    seqs_name <- paste("*target* -", seqs)
                } else {
                    seqs_name <- seqs
                }
                isolates_in_groups[[n]][["groups"]][[seqs_name]] <-
                    paste(names(which(patterns == seqs)), collapse = ", ")
            }
        }
        isolates_in_groups[[n]][["residual"]] <- paste(failed_to_discriminate,
            collapse = ", ")
        isolates_in_groups[[n]][["result"]] <- results[[n]]
        isolates_in_groups[[n]][["N.B.-removed"]] <- names(patterns[which(patterns %in% invalid_pattern)])
        isolates_in_groups[[n]][["N.B.-goi_removed"]] <- intersect(
            names(patterns[which(patterns %in% invalid_pattern)]),
            goi)
        isolates_in_groups[[n]][["N.B.-total removed"]] <- length(names(patterns[which(patterns %in% invalid_pattern)]))
    }
    return(isolates_in_groups)
}

metric_fun <- get_metric_fun()
metric_fun[["percent_non_standard"]] <- list(calc = calculate_percent_remove_non_standard, view = view_percent_remove_non_standard)
metric_fun[["simpson_non_standard"]] <- list(calc = calculate_simpson_remove_non_standard, view = view_simpson_remove_non_standard)
MinSNPs_metrics <- metric_fun